import mitt from 'mitt';
import { reactive } from 'vue';
import { SkinManager } from './skin';
import { NetlistRender } from './render';


export const emitter = mitt();

export const globalSetting = reactive({
    language: 'zh',
    renderAnimation: true
});

export const globalLookup = {
    /**
     * @type {SkinManager}
     */
    skinManager: new SkinManager(),

    /**
     * @type {NetlistRender}
     */
    netlistRender: new NetlistRender(),

    /**
     * @type {number}
     */
    svgScale: 1,

    /**
     * @type {number}
     */
    svgTranslateX: 0,

    /**
     * @type {number}
     */
    svgTranslateY: 0
};

function loadSetting() {
    const settingString = localStorage.getItem('setting')
    try {
        const setting = JSON.parse(settingString);
        return setting;
    } catch (error) {
        return undefined;
    }
}

export const storedSetting = loadSetting();
if (storedSetting) {
    for (const key of Reflect.ownKeys(storedSetting)) {
        const value = storedSetting[key];
        if (value !== undefined) {
            globalSetting[key] = value;
        }
    }
}

