import * as d3 from 'd3';
import { globalSetting } from '../global';

export class CellRender {
    /**
     * 
     * @param {d3.Selection} selection 
     */
    constructor(selection, cells) {
        this.parentSelection = selection;

        /**
         * @type {BasicD3DataItem[]}
         */
        this.data = [];
    }

    /**
     * @description 将 elknode 关于 器件 的数据添加为 d3 数据项目
     * @param {ElkNode} node 
     * @param {HTMLElement} element
     */
    addAsD3DataItem(node, element) {
        this.data.push({
            element,
            x: node.x,
            y: node.y,
            width: node.width,
            height: node.height,
            fill: 'var(--main-dark-color)'
        });
    }

    render() {
        const data = this.data;

        let cellSelections = this.parentSelection.selectAll('g')
            .data(data)
            .enter()
            .append(data => {
                const element = data.element;
                element.setAttribute('x', data.x);
                element.setAttribute('y', data.y);
                if (globalSetting.renderAnimation) {
                    element.setAttribute('stroke-opacity', 0);
                }
                return element;
            });
        
        if (globalSetting.renderAnimation) {
            cellSelections = cellSelections
                .transition()
                .duration(1000)
                .attr('stroke-opacity', 1)
                .attr('class', 'grab')
                .on('end', function (data) {
                    console.log('enter end');
                    
                    const cellSelection = d3.select(this);
                    registerDragEvent(cellSelection, data);
                });
        } else {
            cellSelections = cellSelections
                .attr('class', 'grab')
                .each(function (data) {
                    console.log('enter end'); // 在这里执行你需要的逻辑
                    const cellSelection = d3.select(this);
                    registerDragEvent(cellSelection, data);
                });
        }

        this.selections = cellSelections;
        return cellSelections;
    }
}

/**
 * @description 注册关于 器件 的拖动事件
 * 
 * 需要提取最小拓扑子图，然后重新调整各个区域的尺寸
 * @param {d3.Selection} selection
 * @param {any} data
 */
export function registerDragEvent(selection, data) {
    // 创建拖拽行为
    const drag = d3.drag();

    drag.on('start', event => dragStart(event, selection, data));
    drag.on('drag', event => dragged(event, selection, data));
    drag.on('end', event => dragEnd(event, selection, data))
    
    selection.call(drag);
}

/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 */
function dragStart(event, selection, data) {
    selection.attr('class', 'grabbing');
}

/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 */
function dragged(event, selection, data) {
    data.x = event.x;
    data.y = event.y;
    
    selection
        .attr('x', event.x)
        .attr('y', event.y);
}

/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 */
function dragEnd(event, selection, data) {
    selection.attr('class', 'grab');
}